# Test Functions

def DrawCircle (x, y, r):
	element = APIObject ()
	element.head = APIObject ()
	element.head.typeID = API_ArcID
	element.whole = True
	element.origC = APIObject ()
	element.origC.x = x
	element.origC.y = y
	element.r = r
	return CreateArc (element)

def DrawWall (x1, y1, x2, y2):
	element = APIObject ()
	element.head = APIObject ()
	element.head.typeID = API_WallID
	element.begC = APIObject ()
	element.begC.x = x1
	element.begC.y = y1
	element.endC = APIObject ()
	element.endC.x = x2
	element.endC.y = y2
	return CreateWall (element)

def DrawText (x, y, text):
	element = APIObject ()
	element.head = APIObject ()
	element.head.typeID = API_TextID
	element.head.hasMemo = True
	element.loc = APIObject ()
	element.loc.x = x
	element.loc.y = y
	memo = APIObject ()
	memo.textContent = text
	return CreateText (element, memo)

def MakeDrawing (scale):
	element = APIObject ()
	element.head = APIObject ()
	element.head.typeID = API_DrawingID
	element.head.hasMemo = True
	memo = APIObject ()
	drawingData = DrawingData ()
	drawingData.StartDrawing (scale)
	with UndoScope ("Create Drawing Elements"):
		DrawText (-0.45, 0.2, "Hello")
		DrawCircle (0, 0, 1)
		DrawCircle (0, 0, 2)
		DrawCircle (0, 0, 3)
		DrawCircle (0, 0, 4)
		DrawCircle (0, 0, 5)
	drawingData.StopDrawing ()
	memo.drawingData = drawingData
	element.bounds = drawingData.GetBounds ()
	element.bounds.xMin -= 1
	element.bounds.yMin -= 1
	element.bounds.xMax += 1
	element.bounds.yMax += 1
	element.nameType = APIName_CustomName
	element.name = "Test Drawing"
	return CreateDrawing (element, memo)

def ChangeDrawingTest (guid):
	element = GetDrawing (guid)
	element = element[0]
	element.hasBorderLine = True
	element.borderLineType = 1
	element.borderPen += 2
	element.borderSize *= 1.5
	element.angle = 3.141592654 / 12 # 15 Degrees
	element.colorMode = APIColorMode_GrayScale
	return ChangeDrawing (element)


# Test Fixture

elements = []
elements.append (DrawCircle (5, 5, 5))
elements.append (DrawWall (0, 0, 10, 0))
elements.append (DrawWall (10, 0, 10, 10))
elements.append (DrawText (5, 5, "Hello, world!"))
Alert ("{} Elements Made".format (len (elements)))
DeleteElements (elements)
elements.append (MakeDrawing (100))
Alert ("Drawing Made".format (len (elements)))
ChangeDrawingTest (elements[-1])

# Clean Up

del globals ()["DrawCircle"]
del globals ()["DrawWall"]
del globals ()["DrawText"]
del globals ()["MakeDrawing"]
del globals ()["ChangeDrawingTest"]
del elements
