roomNumberFormat = 'NEW ID {}'
startNumberingToRight = False
zoneGridEpsilon = 2

def find_step (l):
	minD = 0
	step = 0
	sortedList = sorted(set(l))
	for ii in range(len(sortedList) - 1):
		d = abs(sortedList[ii+1] - sortedList[ii])
		if minD == 0 or d < minD:
			minD = d
		if d > zoneGridEpsilon and (step == 0 or d < step):
			step = d
	return minD + zoneGridEpsilon if step == 0 else step

def get_grid_index (pos, first, step):
	return (int)((pos - first) / (1 if step == 0 else step))

def make_grid (zones, startingPos):
	x = [zone.pos.x for zone in zones]
	y = [zone.pos.y for zone in zones]

	minX, minY, stepX, stepY = min(x), min(y), find_step(x), find_step(y)
	maxXIndex, maxYIndex = get_grid_index(max(x), minX, stepX), get_grid_index(max(y), minY, stepY)
	grid = [[None] * (maxXIndex+1) for _ in range(maxYIndex+1)]

	for ii in range(len(x)):
		xIndex, yIndex = get_grid_index(x[ii], minX, stepX), get_grid_index(y[ii], minY, stepY)
		grid[yIndex][xIndex] = zones[ii]

	return (grid, min(maxXIndex, get_grid_index(startingPos.x, minX, stepX)), min(maxYIndex, get_grid_index(startingPos.y, minY, stepY)))

def get_selected_zones ():
	selectedZoneGuids = []
	for elemGuid in GetSelectedElements():
		if GetElementType(elemGuid) == API_ZoneID:
			selectedZoneGuids.append(elemGuid)
	return selectedZoneGuids

def collect_zones ():
	zonesPerFloors, startingPos = {}, None
	selectedZoneGuids = get_selected_zones()
	if not selectedZoneGuids:
		PrintError("Please select at least one zone!")
		return zonesPerFloors, startingPos
	print("Please click one of the selected zones from where to start the numbering!")
	clickedZoneGuid = ClickElement()

	for zoneGuid in selectedZoneGuids:
		zone, memo = GetZone(zoneGuid)
		if zoneGuid == clickedZoneGuid:
			startingPos = zone.pos
		floorKey = (zone.head.floorInd, zone.roomBaseLev)
		if floorKey in zonesPerFloors:
			zonesPerFloors[floorKey].append(zone)
		else:
			zonesPerFloors[floorKey] = [zone]

	return zonesPerFloors, startingPos

def change_zones (zonesPerFloors, startingPos):
	if zonesPerFloors and startingPos is None:
		PrintError("Wrong click!")
		return

	with UndoScope("Changing Zone Numbers"):
		for floorKey in zonesPerFloors.keys():
			grid, xIndexStart, yIndexStart = make_grid(zonesPerFloors[floorKey], startingPos)
			xIndex, yIndex = xIndexStart, yIndexStart
			gridHeight = len(grid)
			gridWidth = len(grid[0])
			gridElemCount = gridWidth * gridHeight
			count = 0
			isDirRight = startNumberingToRight
			for _ in range(gridElemCount):
				zone = grid[yIndex][xIndex]
				if zone is not None:
					count += 1
					zone.roomNoStr = roomNumberFormat.format(count)
					ChangeZone(zone)

				xIndex += (1 if isDirRight else -1)
				if xIndex == (gridWidth if isDirRight else -1):
					yIndex = (yIndex + (1 if startNumberingToRight else -1)) % gridHeight
					isDirRight = (yIndex - yIndexStart) % 2 == (0 if startNumberingToRight else 1)
					xIndex = (0 if isDirRight else gridWidth - 1)

			print("{} zones will be changed on floor {}".format(count, floorKey[0]))

change_zones (*collect_zones ())
